@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2012 Ben Croston
+Copyright (c) 2012-2013 Ben Croston
 
 Permission is hereby granted, free of charge, to any person obtaining a copy of
 this software and associated documentation files (the "Software"), to deal in
@@ -89,7 +89,7 @@ void clear_event_detect(int gpio)
     *(gpio_map+offset) = 0;
 }
 
-int event_detected(int gpio)
+int eventdetected(int gpio)
 {
 	int offset, value, bit;
    
