@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2013 Ben Croston
+Copyright (c) 2013-2014 Ben Croston
 
 Permission is hereby granted, free of charge, to any person obtaining a copy of
 this software and associated documentation files (the "Software"), to deal in
@@ -41,7 +41,7 @@ void define_constants(PyObject *module)
    PyModule_AddObject(module, "IN", input);
 
    pwm = Py_BuildValue("i", PWM);
-   PyModule_AddObject(module, "PWM", pwm);
+   PyModule_AddObject(module, "HARD_PWM", pwm);
 
    serial = Py_BuildValue("i", SERIAL);
    PyModule_AddObject(module, "SERIAL", serial);
@@ -79,6 +79,6 @@ void define_constants(PyObject *module)
    both_edge = Py_BuildValue("i", BOTH_EDGE + PY_EVENT_CONST_OFFSET);
    PyModule_AddObject(module, "BOTH", both_edge);
 
-   version = Py_BuildValue("s", "0.5.4");
+   version = Py_BuildValue("s", "0.5.5");
    PyModule_AddObject(module, "VERSION", version);
 }
